@@ -25,7 +25,7 @@ Created on Apr 20, 2012
 import unittest
 
 from pyclp import *
-
+import os.path,os
     
   
 class Test_atom(unittest.TestCase):
@@ -203,7 +203,44 @@ class Test_PList(unittest.TestCase):
         for i in range(len(input_tuple)):
             self.assertEqual(input_tuple[i],returned_value[i])
             self.assertEqual(input_tuple[-i],returned_value[-i])
-
+    def testNestedList(self):
+        my_var=Var()
+        myList=PList([1,2,3,["ciao"]])
+        Compound("=",my_var,myList).post_goal()
+        self.assertTrue(resume())
+        self.assertEqual(str(my_var.value()), '[1,2,3,["ciao"]]', "Incorrect conversion to string")
+    def testNotNullTail(self):
+        """
+        Test when the tail is not the nil list -->
+        """
+        myVar=Var()
+        myList=PList([1,2,3],tail=Atom("test"))
+        self.assertEqual(str(myList),'[1,2,3|test]',"Failed conversion to str")
+        myList=PList([1,2,3],tail=myVar)
+        self.assertEqual(str(myList), '[1,2,3|_]',"Failed conversion to str")
+        myCompound=Compound('f',"ciao",1)
+        Compound('=',myList,PList([1,2,3],tail=myCompound)).post_goal()
+        resume()
+        self.assertEqual(str(myVar.value()), 'f("ciao",1)',"Failed unit test")
+        #test getListTail
+        self.assertEqual(myList.getListTail(), ([1,2,3],myCompound), "Failed test of getListTail")
+        #test get index
+        self.assertEqual(myList[-1], 3, "Failed getitem with negative index")
+        self.assertEqual(myList[-3], 1, "Failed getitem with negative index")
+        #test len see that tail is not considered
+        self.assertEqual(len(myList),3,"Failed len")
+        #Test iterHeadTail
+        headList=[]
+        tailList=[]
+        for head,tail in myList.iterHeadTail():
+            headList.append(head)
+            tailList.append(tail)
+        self.assertEqual(headList, [1,2,3], "Incorrect head conversion generated by iterHeadTail")
+        referenceTailList=[PList([2,3],tail=myCompound),PList([3],tail=myCompound),myCompound]
+        self.assertEqual(tailList,referenceTailList, "Incorrect tail conversion generated by iterHeadTail")
+            
+        
+        
         
         
 class Test_Stream(unittest.TestCase):
@@ -355,8 +392,7 @@ class Test_initialization(unittest.TestCase):
         cleanup()
         with self.assertRaises(pyclpEx):
             cleanup()
-   
-        
+       
      
 class Test_external_predicates(unittest.TestCase):
     """
@@ -392,6 +428,28 @@ class Test_external_predicates(unittest.TestCase):
         Compound('call_python_function',Atom(eclipse_name),[1,my_var]).post_goal()
         with self.assertRaises(UnboundLocalError) as exp:
             resume()
+    def test_call_other_module(self):
+        """
+        Test external execution using a eclipse module to test 
+        that call_python_function is correctly exported.
+        """
+        runnig_directory=os.getcwd() 
+        os.chdir(os.path.dirname(os.path.realpath(__file__)))
+        Compound("use_module",Atom("test")).post_goal()
+        self.assertEqual(resume(),(SUCCEED,None),"Failed importing eclipse module test ")
+        eclipse_name='p_func'
+        
+        def called_from_eclipse(arguments):
+            return unify(arguments[0],arguments[1])
+        
+        addPythonFunction(eclipse_name,called_from_eclipse)
+        my_var=Var()
+        Compound('test',1,my_var).post_goal()
+        self.assertEqual(resume(),(SUCCEED,None),"Failed resume ")
+        self.assertEqual(my_var.value(),1,"Failed unification")
+        os.chdir(runnig_directory)
+        
+       
         
         
 
